﻿using DragDropBetweenTreeViewAndListBox.ViewModels;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Timers;
using System.Windows;
using System.Linq;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.TreeView;
using Telerik.Windows.DragDrop;

namespace DragDropBetweenTreeViewAndListBox
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            this.TreeView1.ItemsSource = DataItem.Generate(5, 3);
            this.TreeView2.ItemsSource = DataItem.Generate(3, 2);

            DragDropManager.AddDropHandler(TreeView2, OnDrop, true);

            // Add DragOver event handled on the first RadTreeView to disable drop in it.
            DragDropManager.AddDragOverHandler(TreeView1, OnDragOver, true);
        }

        private void OnDragOver(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
        {
            var options = DragDropPayloadManager.GetDataFromObject(e.Data, TreeViewDragDropOptions.Key) as TreeViewDragDropOptions;
            options.DropAction = DropAction.None;
        }

       
        private void OnDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
        {
            var dropDetails = DragDropPayloadManager.GetDataFromObject(e.Data, TreeViewDragDropOptions.Key) as TreeViewDragDropOptions;

            var radTreeView = sender as RadTreeView;
            
            var draggedSourceItem = dropDetails.DragSourceItem as RadTreeViewItem;
            var draggedDataItem = dropDetails.DraggedItems.FirstOrDefault() as DataItem;

            RadTreeViewItem draggedElementParent = draggedSourceItem.ParentItem as RadTreeViewItem;

            if (draggedElementParent != null) // if the element has parent
            {
                DataItem draggedItemParent = draggedElementParent.Item as DataItem;

                DataItem treeDataItem = ((ObservableCollection<DataItem>)radTreeView.ItemsSource).FirstOrDefault(x => x.Header == draggedItemParent.Header);
                if (treeDataItem != null) // if there is a parent already - add only the child
                {
                    if (!treeDataItem.Children.Any(x => x.Header == draggedDataItem.Header))
                    {
                        treeDataItem.Children.Add(draggedDataItem);
                    }
                }
                else // if there is no parent - add only the parent and the selected child in it
                {
                    draggedItemParent = new DataItem { Header = draggedItemParent.Header };
                    draggedItemParent.Children.Add(draggedDataItem);

                    ((IList)radTreeView.ItemsSource).Add(draggedItemParent);
                }
            }
            else //if the dragged item is a parent node - add it in the tree with all its children
            {
                bool containsParent = ((ObservableCollection<DataItem>)radTreeView.ItemsSource).Any(x => x.Header == draggedDataItem.Header);
                if (!containsParent)
                {
                    ((IList)radTreeView.ItemsSource).Add(draggedDataItem);
                }
            }

            // Clear the dragged items
            ((IList)dropDetails.DraggedItems).Clear();
        }
    }
}
